@@ -4,41 +4,16 @@ Representation of items in a Doorstop document.
 
 import os
 import re
-import functools
 import logging
 
-import yaml
-
+from doorstop.core.base import auto_load, auto_save, BaseFileObject, Literal
 from doorstop import common
 from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
-from doorstop.settings import SEP_CHARS
+from doorstop import settings
 
 
-def auto_load(func):
-    """Decorator for methods that should automatically load from file."""
-    @functools.wraps(func)
-    def wrapped(self, *args, **kwargs):
-        """Wrapped method to call self.load() before execution."""
-        self.load()
-        return func(self, *args, **kwargs)
-    return wrapped
-
-
-def auto_save(func):
-    """Decorator for methods that should automatically save to file."""
-    @functools.wraps(func)
-    def wrapped(self, *args, **kwargs):
-        """Wrapped method to call self.save() after execution."""
-        result = func(self, *args, **kwargs)
-        if self.auto:
-            self.save()
-        return result
-
-    return wrapped
-
-
-class Item(object):  # pylint: disable=R0904
-    """Represents a file with linkable text that is part of a document."""
+class Item(BaseFileObject):  # pylint: disable=R0904
+    """Represents an item file with linkable text."""
 
     EXTENSIONS = '.yml', '.yaml'
 
@@ -49,8 +24,6 @@ class Item(object):  # pylint: disable=R0904
     DEFAULT_TEXT = ""
     DEFAULT_REF = ""
 
-    auto = True  # set to False to delay automatic save until explicit save
-
     def __init__(self, path, root=os.getcwd()):
         """Load an item from an existing file.
 
@@ -60,6 +33,7 @@ class Item(object):  # pylint: disable=R0904
         @param path: path to Item file
         @param root: path to root of project
         """
+        super().__init__()
         # Ensure the path is valid
         if not os.path.isfile(path):
             raise DoorstopError("item does not exist: {}".format(path))
@@ -75,12 +49,10 @@ class Item(object):  # pylint: disable=R0904
         if ext.lower() not in self.EXTENSIONS:
             msg = "'{0}' extension not in {1}".format(path, self.EXTENSIONS)
             raise DoorstopError(msg)
-        # Initialize Item
+        # Initialize the item
         self.path = path
         self.root = root
-        self._exists = True
-        self._data = {}
-        # Set defaults
+        # Set default values
         self._data['level'] = Item.DEFAULT_LEVEL
         self._data['active'] = Item.DEFAULT_ACTIVE
         self._data['normative'] = Item.DEFAULT_NORMATIVE
@@ -93,13 +65,13 @@ class Item(object):  # pylint: disable=R0904
         return "Item({})".format(repr(self.path))
 
     def __str__(self):
-        if common.VERBOSITY <= 1:
+        if common.VERBOSITY < common.STR_VERBOSITY:
             return self.id
         else:
             return self.id_relpath
 
     def __eq__(self, other):
-        return isinstance(other, Item) and self.path == other.path
+        return isinstance(other, self.__class__) and self.path == other.path
 
     def __ne__(self, other):
         return not self == other
@@ -108,15 +80,15 @@ class Item(object):  # pylint: disable=R0904
         return self.level < other.level
 
     @staticmethod
-    def new(path, root, prefix, sep, number, digits, level, auto=None):  # pylint: disable=R0913
+    def new(path, root, prefix, sep, digits, number, level, auto=None):  # pylint: disable=R0913
         """Create a new item.
 
         @param path: path to directory for the new item
         @param root: path to root of the project
         @param prefix: prefix for the new item
         @param sep: separator between prefix and number
-        @param number: number for the new item
         @param digits: number of digits for the new document
+        @param number: number for the new item
         @param level: level for the new item (None for default)
         @param auto: enables automatic save
 
@@ -127,39 +99,25 @@ class Item(object):  # pylint: disable=R0904
         path2 = os.path.join(path, filename)
         # Create the initial item file
         logging.debug("creating item file at {}...".format(path2))
-        Item._new(path2)
+        Item._new(path2, name='item')
         # Initialize the item
         item = Item(path2, root=root)
         item.auto = False
         item.level = level or Item.DEFAULT_LEVEL
-        item.auto = Item.auto if auto is None else auto
-        # Return the new item
+        if auto or (auto is None and Item.auto):
+            item.save()
+        # Return the item
         return item
 
-    @staticmethod
-    def _new(path):  # pragma: no cover, integration test
-        """Create a new item file.
-
-        @param config: path to new item file
-        """
-        if os.path.exists(path):
-            raise DoorstopError("item already exists: {}".format(path))
-        with open(path, 'w'):
-            pass  # just touch the file
-
     def load(self, reload=False):
-        """Load the item's properties from a file."""
-        if getattr(self, '_loaded', False) and not reload:
+        """Load the item's properties from its file."""
+        if self._loaded and not reload:
             return
         logging.debug("loading {}...".format(repr(self)))
-        # Read the YAML from file
-        text = self._read()
-        # Parse the YAML data
-        try:
-            data = yaml.load(text) or {}
-        except yaml.scanner.ScannerError as error:  # pylint: disable=E1101
-            msg = "invalid contents: {}:\n{}".format(self, error)
-            raise DoorstopError(msg)
+        # Read text from file
+        text = self._read(self.path)
+        # Parse YAML data from text
+        data = self._load(text, self.path)
         # Store parsed data
         for key, value in data.items():
             if key == 'level':
@@ -179,24 +137,19 @@ class Item(object):  # pylint: disable=R0904
             else:
                 self._data[key] = value
         # Set meta attributes
-        setattr(self, '_loaded', True)
-
-    def _read(self):  # pragma: no cover, integration test
-        """Read text from the file."""
-        if not self._exists:
-            raise DoorstopError("cannot load from deleted: {}".format(self))
-        with open(self.path, 'rb') as infile:
-            return infile.read().decode('UTF-8')
+        self._loaded = True
 
     def save(self):
-        """Format and save the item's properties to a file."""
+        """Format and save the item's properties to its file."""
         logging.debug("saving {}...".format(repr(self)))
         # Format the data items
         data = {}
         for key, value in self._data.items():
             if key == 'level':
                 level = '.'.join(str(n) for n in value)
-                if len(value) == 2:
+                if len(value) == 1:
+                    level = int(level)
+                elif len(value) == 2:
                     level = float(level)
                 data['level'] = level
             elif key == 'text':
@@ -206,23 +159,17 @@ class Item(object):  # pylint: disable=R0904
             elif key == 'links':
                 data['links'] = sorted(value)
             else:
+                # TODO: dump long strings as Literal (and SBD?)
                 data[key] = value
         # Dump the data to YAML
-        dump = yaml.dump(data, default_flow_style=False)
+        text = self._dump(data)
         # Save the YAML to file
-        self._write(dump)
+        self._write(text, self.path)
         # Set meta attributes
-        setattr(self, '_loaded', False)
+        self._loaded = False
         self.auto = True
 
-    def _write(self, text):  # pragma: no cover, integration test
-        """Write text to the file."""
-        if not self._exists:
-            raise DoorstopError("cannot save to deleted: {}".format(self))
-        with open(self.path, 'wb') as outfile:
-            outfile.write(bytes(text, 'UTF-8'))
-
-    # standard attributes ####################################################
+    # properties #############################################################
 
     @property
     def id(self):  # pylint: disable=C0103
@@ -238,7 +185,7 @@ class Item(object):  # pylint: disable=R0904
     # TODO: think of a better name for this property
     @property
     def id_relpath(self):
-        """Get the item's ID and relative path string."""
+        """Get the item's ID + relative path string."""
         return "{} ({})".format(self.id, self.relpath)
 
     @property
@@ -254,18 +201,18 @@ class Item(object):  # pylint: disable=R0904
     @property
     @auto_load
     def level(self):
-        """Get the item level."""
+        """Get the item's level."""
         return self._data['level']
 
     @level.setter
     @auto_save
-    def level(self, level):
+    def level(self, value):
         """Set the item's level."""
-        self._data['level'] = convert_level(level)
+        self._data['level'] = convert_level(value)
 
     @property
     def depth(self):
-        """Get the heading order based on the level."""
+        """Get the item's heading order based on it's level."""
         level = list(self.level)
         while level[-1] == 0:
             del level[-1]
@@ -274,21 +221,40 @@ class Item(object):  # pylint: disable=R0904
     @property
     @auto_load
     def active(self):
-        """Indicates the item should be considered for linking.
+        """Get the item's active status.
 
-        Inactive items are intended to be used for:
+        An inactive item will not be validated. Inactive items are
+        intended to be used for:
          - future requirements
          - temporarily disabled requirements or tests
          - externally implemented requirements
+         - etc.
 
         """
         return self._data['active']
 
     @active.setter
     @auto_save
-    def active(self, status):
+    def active(self, value):
         """Set the item's active status."""
-        self._data['active'] = bool(status)
+        self._data['active'] = bool(value)
+
+    @property
+    @auto_load
+    def derived(self):
+        """Get the item's derived status.
+
+        A derived item does not have links to items in its parent
+        document, but should still be linked to by items in its child
+        documents.
+        """
+        return self._data['derived']
+
+    @derived.setter
+    @auto_save
+    def derived(self, value):
+        """Set the item's derived status."""
+        self._data['derived'] = bool(value)
 
     @property
     @auto_load
@@ -299,38 +265,37 @@ class Item(object):  # pylint: disable=R0904
         Non-normative items are intended to be used for:
          - headings
          - comments
+         - etc.
 
         """
         return self._data['normative']
 
     @normative.setter
     @auto_save
-    def normative(self, status):
+    def normative(self, value):
         """Set the item's normative status."""
-        self._data['normative'] = bool(status)
+        self._data['normative'] = bool(value)
 
     @property
-    @auto_load
-    def derived(self):
-        """Get the item's derived status.
+    def heading(self):
+        """Indicates if the item is a heading.
 
-        A derived item does not have links to items in its parent
-        document, but should still be linked to by items in its child
-        document.
+        Headings have a level that ends in zero and are non-normative.
         """
-        return self._data['derived']
-
-    @derived.setter
-    @auto_save
-    def derived(self, status):
-        """Set the item's derived status."""
-        self._data['derived'] = bool(status)
-
-    @property
-    def header(self):
-        """Indicates if the item is a header."""
         return self.level[-1] == 0 and not self.normative
 
+    @heading.setter
+    @auto_save
+    def heading(self, value):
+        """Set the item's heading status."""
+        heading = bool(value)
+        if heading and not self.heading:
+            self.level = list(self.level) + [0]
+            self.normative = False
+        elif not heading and self.heading:
+            self.level = list(self.level)[:-1]
+            self.normative = True
+
     @property
     @auto_load
     def text(self):
@@ -339,89 +304,67 @@ class Item(object):  # pylint: disable=R0904
 
     @text.setter
     @auto_save
-    def text(self, text):
+    def text(self, value):
         """Set the item's text."""
-        self._data['text'] = text
+        self._data['text'] = value
 
     @property
     @auto_load
     def ref(self):
-        """Get the item's external file reference."""
+        """Get the item's external file reference.
+
+        An external reference can be part of a line in a text file or
+        the filename of any type of file.
+        """
         return self._data['ref']
 
     @ref.setter
     @auto_save
-    def ref(self, ref):
+    def ref(self, value):
         """Set the item's external file reference."""
-        self._data['ref'] = ref
+        self._data['ref'] = value
 
     @property
     @auto_load
     def links(self):
-        """Get the items this item links to."""
+        """Get a list of the item IDs this item links to."""
         return sorted(self._data['links'])
 
     @links.setter
     @auto_save
-    def links(self, links):
-        """Set the items this item links to."""
-        self._data['links'] = set(links)
-
-    # extended attributes ####################################################
-
-    @auto_load
-    def get(self, name, default=None):
-        """Get an extended attribute."""
-        if hasattr(self, name):
-            cname = self.__class__.__name__
-            msg = "'{n}' can be accessed from {c}.{n}".format(n=name, c=cname)
-            logging.info(msg)
-            return getattr(self, name)
-        else:
-            return self._data.get(name, default)
-
-    @auto_load
-    @auto_save
-    def set(self, name, value):
-        """Set an extended attribute."""
-        if hasattr(self, name):
-            cname = self.__class__.__name__
-            msg = "'{n}' can be set from {c}.{n}".format(n=name, c=cname)
-            logging.info(msg)
-            return setattr(self, name, value)
-        else:
-            self._data[name] = value
+    def links(self, value):
+        """Set the list of item IDs this item links to."""
+        self._data['links'] = set(value)
 
     # actions ################################################################
 
     @auto_load
     @auto_save
     def add_link(self, item):
-        """Add a new link to another item."""
+        """Add a new link to another item ID."""
         self._data['links'].add(item)
 
     @auto_load
     @auto_save
     def remove_link(self, item):
-        """Remove an existing link."""
+        """Remove an existing link by item ID."""
         try:
             self._data['links'].remove(item)
         except KeyError:
             logging.warning("link to {0} does not exist".format(item))
 
-    def valid(self, document=None, tree=None, ignored=None):
+    def valid(self, document=None, tree=None):
         """Check the item for validity.
 
-        @param document: document to validate the item
-        @param tree: tree to validate the item
-        @param ignored: function to determine if a path should be skipped
+        @param document: Document containing the item
+        @param tree: Tree containing the item
 
         @return: indication that the item is valid
         """
+        # TODO: this could be common code with Item/Document/Tree
         valid = True
         # Display all issues
-        for issue in self.iter_issues(document=document, tree=tree,
-                                      ignored=ignored):
+        for issue in self.issues(document=document, tree=tree):
             if isinstance(issue, DoorstopInfo):
                 logging.info(issue)
             elif isinstance(issue, DoorstopWarning):
@@ -433,12 +376,11 @@ class Item(object):  # pylint: disable=R0904
         # Return the result
         return valid
 
-    def iter_issues(self, document=None, tree=None, ignored=None):
+    def issues(self, document=None, tree=None):
         """Yield all the item's issues.
 
-        @param document: document to validate the item
-        @param tree: tree to validate the item
-        @param ignored: function to determine if a path should be skipped
+        @param document: Document containing the item
+        @param tree: Tree containing the item
 
         @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
         """
@@ -455,7 +397,7 @@ class Item(object):  # pylint: disable=R0904
             yield DoorstopWarning("no text")
         # Check external references
         try:
-            self.find_ref(ignored=ignored)
+            self.find_ref(ignored=tree.vcs.ignored if tree else None)
         except DoorstopError as exc:
             yield exc
         # Check links
@@ -463,15 +405,22 @@ class Item(object):  # pylint: disable=R0904
             yield DoorstopWarning("non-normative, but has links")
         # Check links against the document
         if document:
-            yield from self._iter_issues_document(document)
+            yield from self._issues_document(document)
         # Check links against the tree
         if tree:
-            yield from self._iter_issues_tree(tree)
+            yield from self._issues_tree(tree)
+        # Check links against both document and tree
+        if document and tree:
+            yield from self._issues_both(document, tree)
         # Reformat the file
         self.save()
 
-    def _iter_issues_document(self, document):
+    def _issues_document(self, document):
         """Yield all the item's issues against its document."""
+        # Verify an item's ID matches its document's prefix
+        if self.prefix != document.prefix:
+            msg = "prefix differs from document ({})".format(document.prefix)
+            yield DoorstopInfo(msg)
         # Verify an item has upward links
         if all((document.parent,
                 self.normative,
@@ -480,13 +429,21 @@ class Item(object):  # pylint: disable=R0904
             yield DoorstopWarning(msg)
         # Verify an item's links are to the correct parent
         for identifier in self.links:
-            prefix = split_id(identifier)[0]
-            if prefix.lower() != document.parent.lower():
-                msg = "linked to non-parent item: {}".format(identifier)
-                yield DoorstopInfo(msg)
+            try:
+                prefix = split_id(identifier)[0]
+            except DoorstopError:
+                msg = "invalid ID in links: {}".format(identifier)
+                yield DoorstopError(msg)
+            else:
+                if prefix != document.parent:
+                    # this is only 'info' because a document is allowed
+                    # to contain items with a different prefix, but
+                    # Doorstop will not create items like this
+                    msg = "linked to non-parent item: {}".format(identifier)
+                    yield DoorstopInfo(msg)
 
-    def _iter_issues_tree(self, tree):
-        """Yield all the item's issues against the full tree."""
+    def _issues_tree(self, tree):
+        """Yield all the item's issues against its tree."""
         # Verify an item's links are valid
         identifiers = set()
         for identifier in self.links:
@@ -508,25 +465,19 @@ class Item(object):  # pylint: disable=R0904
                 identifiers.add(identifier)
         # Apply the reformatted item IDs
         self._data['links'] = identifiers
+
+    def _issues_both(self, document, tree):
+        """Yield all the item's issues against its document and tree."""
         # Verify an item is being linked to (reverse links)
-        rlinks = []
-        children = []
-        for document in tree:
-            if document.parent == self.prefix:
-                children.append(document)
-                for item in document:
-                    if self.id in item.links:
-                        rlinks.append(item.id)
-        if rlinks:
-            msg = "reverse links: {}".format(', '.join(rlinks))
-            logging.debug(msg)
-        elif self.normative:
-            for child in children:
-                msg = "no links from child document: {}".format(child)
-                yield DoorstopWarning(msg)
+        if self.normative:
+            rlinks, children = self.find_rlinks(document, tree, find_all=False)
+            if not rlinks:
+                for child in children:
+                    msg = "no links from child document: {}".format(child)
+                    yield DoorstopWarning(msg)
 
     def find_ref(self, root=None, ignored=None):
-        """Find the external file reference and line number.
+        """Get the external file reference and line number.
 
         @param root: override the path to the working copy (for testing)
         @param ignored: function to determine if a path should be skipped
@@ -557,6 +508,8 @@ class Item(object):  # pylint: disable=R0904
                     continue
                 # Skip ignored paths
                 if ignored(path):
+                    # TODO: remove this logging line
+                    logging.debug("skipped: {}".format(path))
                     continue
                 # Search for the reference in the file
                 if filename == self.ref:
@@ -572,25 +525,38 @@ class Item(object):  # pylint: disable=R0904
         msg = "external reference not found: {}".format(self.ref)
         raise DoorstopError(msg)
 
-    def delete(self):
-        """Delete the item from the file system."""
-        logging.info("deleting {}...".format(self.path))
-        os.remove(self.path)
-        self._exists = False  # prevent future access
+    # TODO: should this only return IDs and add a find_children method?
+    def find_rlinks(self, document, tree, find_all=True):
+        """Get a list of item IDs that link to this item (reverse links).
 
+        @param document: Document containing the item
+        @param tree: Tree containing the item
+        @param find_all: find all items (not just the first) before returning
 
-# YAML representer classes ###################################################
+        @return: list of found item IDs, list of all child Documents
+        """
+        rlinks = []
+        children = []
+        for document2 in tree:
+            if document2.parent == document.prefix:
+                children.append(document2)
+            # Search for reverse links unless we only need to find one
+            if not rlinks or find_all:
+                for item in document2:
+                    if self.id in item.links:
+                        rlinks.append(item.id)
+                        if not find_all:
+                            break
+        if rlinks:
+            if find_all:
+                msg = "reverse links: {}".format(', '.join(rlinks))
+            else:
+                msg = "first reverse link: {}".format(rlinks[0])
+            logging.debug(msg)
+        return rlinks, children
 
-class Literal(str):  # pylint: disable=R0904
-    """Custom type for text which should be dumped in the literal style."""
-
-    @staticmethod
-    def representer(dumper, data):
-        """Return a custom dumper that formats str in the literal style."""
-        return dumper.represent_scalar('tag:yaml.org,2002:str', data,
-                                       style='|' if data else '')
-
-yaml.add_representer(Literal, Literal.representer)
+    def delete(self, path=None):
+        super().delete(self.path)
 
 
 # attribute formatters #######################################################
@@ -604,11 +570,14 @@ def split_id(text):
     >>> split_id('ABC.HLR_01-00123')
     ('ABC.HLR_01', 123)
 
+    >>> split_id('REQ2-001')
+    ('REQ2', 1)
+
     """
     match = re.match(r"([\w.-]*\D)(\d+)", text)
     if not match:
         raise DoorstopError("invalid ID: {}".format(text))
-    prefix = match.group(1).rstrip(SEP_CHARS)
+    prefix = match.group(1).rstrip(settings.SEP_CHARS)
     number = int(match.group(2))
     return prefix, number
 
@@ -641,6 +610,9 @@ def convert_level(text):
     >>> convert_level([7, 0, 0])
     (7, 0)
 
+    >>> convert_level(1)
+    (1,)
+
     """
     # Correct for integers (42) and floats (4.2) in YAML
     if isinstance(text, int) or isinstance(text, float):
@@ -656,9 +628,6 @@ def convert_level(text):
         while parts[-1] == 0:
             del parts[-1]
         parts.append(0)
-    # Ensure the top level always a header (ends in a zero)
-    if len(parts) == 1:
-        parts.append(0)
     # Convert the level to a tuple
     return tuple(parts)
 
