@@ -10,12 +10,11 @@ import os
 from doorstop.core import Item
 from doorstop.core import Document
 from doorstop.core import Tree, build
-from doorstop.common import DoorstopError
 
 from doorstop.core.test import ENV, REASON, ROOT, FILES, EMPTY
 
 
-class DocumentNoSkip(Document):
+class DocumentNoSkip(Document):  # pylint: disable=R0904
     """Document class that is never skipped."""
 
     SKIP = '__disabled__'  # never skip test Documents
@@ -61,7 +60,7 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
         doc = Document(FILES)
         self.assertEqual('REQ', doc.prefix)
         self.assertEqual(2, doc.digits)
-        self.assertEqual(3, len(doc.items))
+        self.assertEqual(5, len(doc.items))
 
     def test_new(self):
         """Verify a new document can be created."""
@@ -86,10 +85,10 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
             item.write(self.backup)
 
     @patch('doorstop.core.document.Document', DocumentNoSkip)
-    def test_check_invalid_link(self):
+    def test_valid_invalid_link(self):
         """Verify a tree is invalid with a bad link."""
         item = Item(self.ITEM)
         item.add_link('SYS003')
         tree = build(FILES, root=FILES)
         self.assertIsInstance(tree, Tree)
-        self.assertRaises(DoorstopError, tree.check)
+        self.assertFalse(tree.valid())
