@@ -12,39 +12,22 @@ import tempfile
 import operator
 import logging
 
-from doorstop.common import DoorstopError
+from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 from doorstop.core.tree import Tree, build
 from doorstop.core.document import Document
 
 from doorstop.core.test import ENV, REASON, FILES, SYS, EMPTY
+from doorstop.core.test.test_document import MockDocument as _MockDocment
 
 
-class MockDocument(Document):
-    """Mock Document class that does not touch the file system."""
+class MockDocument(_MockDocment):  # pylint: disable=W0223,R0902,R0904
+    """Mock Document class that is always skipped in tree placement."""
 
-    @patch('os.path.isfile', Mock(return_value=True))
-    def __init__(self, *args, **kwargs):
-        self._file = ""  # file system mock
-        self._read = Mock(side_effect=self._mock_read)
-        self._write = Mock(side_effect=self._mock_write)
-        super().__init__(*args, **kwargs)
-
-    def _mock_read(self):
-        """Mock read function."""
-        text = self._file
-        logging.debug("mock read: {0}".format(repr(text)))
-        return text
-
-    def _mock_write(self, text):
-        """Mock write function"""
-        logging.debug("mock write: {0}".format(repr(text)))
-        self._file = text
-
-    _new = Mock()
+    skip = True
 
 
-class MockDocumentNoSkip(MockDocument):
-    """Mock Document class that does not touch the file system."""
+class MockDocumentNoSkip(MockDocument):  # pylint: disable=W0223,R0902,R0904
+    """Mock Document class that is never skipped in tree placement."""
 
     SKIP = '__disabled__'  # never skip mock Documents
 
@@ -64,11 +47,6 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         b2.children = [c1, c2]
         cls.tree = a
 
-    def test_repr(self):
-        """Verify trees can be represented."""
-        text = "<Tree a <- [ b1, b2 <- [ c1, c2 ] ]>"
-        self.assertEqual(text, repr(self.tree))
-
     def test_str(self):
         """Verify trees can be converted to strings."""
         text = "a <- [ b1, b2 <- [ c1, c2 ] ]"
@@ -100,7 +78,7 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         docs = [a, b, c]
         tree = Tree.from_list(docs)
         self.assertEqual(3, len(tree))
-        tree.check()
+        self.assertTrue(tree.valid())
 
     def test_from_list_no_root(self):
         """Verify an error occurs when the tree has no root."""
@@ -109,6 +87,13 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
         docs = [a, b]
         self.assertRaises(DoorstopError, Tree.from_list, docs)
 
+    def test_from_list_multiple_roots(self):
+        """Verify an error occurs when the tree has multiple roots."""
+        a = MockDocument(EMPTY, _prefix='A')
+        b = MockDocument(EMPTY, _prefix='B')
+        docs = [a, b]
+        self.assertRaises(DoorstopError, Tree.from_list, docs)
+
     def test_from_list_missing_parent(self):
         """Verify an error occurs when a node has a missing parent."""
 
@@ -144,18 +129,31 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
                                parent='REQ')
         self.assertRaises(DoorstopError, tree._place, doc)  # pylint: disable=W0212
 
-    @patch('doorstop.core.document.Document.check')
-    def test_check(self, mock_check):
-        """Verify document trees can be checked."""
+    @patch('doorstop.core.document.Document.issues')
+    def test_valid(self, mock_issues):
+        """Verify trees can be checked."""
         logging.info("tree: {}".format(self.tree))
-        self.tree.check()
-        self.assertEqual(2, mock_check.call_count)
+        self.assertTrue(self.tree.valid())
+        self.assertEqual(2, mock_issues.call_count)
 
     @unittest.skipUnless(os.getenv(ENV), REASON)
-    def test_check_long(self):
-        """Verify document trees can be checked (long)."""
+    def test_valid_long(self):
+        """Verify trees can be checked (long)."""
         logging.info("tree: {}".format(self.tree))
-        self.tree.check()
+        self.assertTrue(self.tree.valid())
+
+    def test_valid_no_documents(self):
+        """Verify an empty tree can be checked."""
+        tree = Tree(None, root='.')
+        self.assertTrue(tree.valid())
+
+    def test_valid_document(self):
+        """Verify an document error fails the tree valid."""
+        mock_issues = Mock(return_value=[DoorstopError('e'),
+                                         DoorstopWarning('w'),
+                                         DoorstopInfo('i')])
+        with patch.object(self.tree, 'issues', mock_issues):
+            self.assertFalse(self.tree.valid())
 
     def test_new(self):
         """Verify a new document can be created on a tree."""
@@ -168,7 +166,7 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
                           temp, '_TEST', parent='UNKNOWN')
         self.assertFalse(os.path.exists(temp))
 
-    @patch('doorstop.core.vcs.veracity.WorkingCopy.lock')
+    @patch('doorstop.core.vcs.git.WorkingCopy.lock')
     @patch('doorstop.core.document.Document.add')
     def test_add(self, mock_add, mock_lock):
         """Verify an item can be added to a document."""
@@ -235,7 +233,7 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
         """Verify an exception is raised with an unknown parent prefix."""
         self.assertRaises(DoorstopError, self.tree.unlink, 'req3', 'req9999')
 
-    @patch('doorstop.core.vcs.veracity.WorkingCopy.lock')
+    @patch('doorstop.core.vcs.git.WorkingCopy.lock')
     @patch('doorstop.core.tree._open')
     def test_edit(self, mock_open, mock_lock):
         """Verify an item can be edited in a tree."""
